module TreasureHunt
  require 'rspec'
  require 'treasure_game/game'

  RSpec.describe Game do

    before do
      @game = Game.new('Blokes')
      @initial_health = 100 
      @player = Player.new('moe', @initial_health)
      @game.add_player(@player)
    end

    it 'heals the player if a high number is rolled' do
      allow_any_instance_of(Die).to receive(:roll).and_return(5)
      @game.play(2)
      expect(@player.health).to eq(@initial_health + (15 * 2))
    end

    it 'should do nothing if a medium number is rolled' do
      allow_any_instance_of(Die).to receive(:roll).and_return(3)
      @game.play(2)
      expect(@player.health).to eq(@initial_health)
    end

    it 'hits the player if a low number is rolled' do
      allow_any_instance_of(Die).to receive(:roll).and_return(2)
      @game.play(2)
      expect(@player.health).to eq(@initial_health - (10 * 2))
    end

    context 'tallying up all points to display total points' do
      before do
        @game = Game.new("Knuckleheads")
        @player1 = Player.new("moe")
        @player2 = Player.new("larry")
        @game.add_player(@player1)
        @game.add_player(@player2)
        @player1.found_treasure(Treasure.new(:guitar, 50))
        @player1.found_treasure(Treasure.new(:guitar, 50))
        @player2.found_treasure(Treasure.new(:ruby, 400))
      end

      it "computes total points as the sum of all player points" do
        expect(@game.total_points).to eq(500)
      end
    end

  end
end