module TreasureHunt
  require 'rspec'
  require 'treasure_game/player'
  require 'treasure_game/treasure_hunt'

  RSpec.describe Player do

    before do
      @player = Player.new('mike', 150)
      @h = @player.health
      $stdout = StringIO.new
    end

    it 'has a capitalized name' do
      expect(@player.name).to eq('Mike')
    end

    it 'has an initial health' do
      expect(@h).to eq(150)
    end

    it 'has a string representation' do
      expect(@player.to_s).to eq('Mike 150hp')
    end

    it 'computes a score as the sum of its health and points' do
      @player.found_treasure(Treasure.new(:guitar, 50))
      @player.found_treasure(Treasure.new(:guitar, 50))
      expect(@player.score).to eq(250)
    end


    it 'increases health by 15 when healed' do
      @player.heal
      expect(@player.health).to eq(@h + 15)
    end

    it 'decreases health by 10 when hit' do
      @player.hit
      expect(@player.health).to eq(@h - 10)
    end

    it 'computes points as the sum of all treasure points' do
      expect(@player.points).to eq(0)
      @player.found_treasure(Treasure.new(:guitar, 50))
      expect(@player.points).to eq(50)
      @player.found_treasure(Treasure.new(:ruby, 400))
      expect(@player.points).to eq(450)
      @player.found_treasure(Treasure.new(:guitar, 50))
      expect(@player.points).to eq(500)
    end

    it 'yields each found treasure and its total points' do
      @player.found_treasure(Treasure.new(:cash, 100))
      @player.found_treasure(Treasure.new(:cash, 100))
      @player.found_treasure(Treasure.new(:guitar, 50))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))

      yielded = []
      @player.each_found_treasure do |treasure|
        yielded << treasure
      end

      expect(yielded).to eq([
        Treasure.new(:cash, 200),
        Treasure.new(:guitar, 50),
        Treasure.new(:bottle, 25)
      ])
    end
    
    context 'with a health greater than 100' do
      before do
        @player = Player.new('mike', 150)
      end

      it 'is strong' do
        expect(@player.strong?).to eq(true)
      end
    end

    context 'with a health less than 100' do
      before do
        @player = Player.new('mike', 50)
      end

      it 'is weak' do
        expect(@player.strong?).to eq(false)
      end
    end

    context 'in a collection of players' do
      before do
        @player1 = Player.new('mike', 100)
        @player2 = Player.new('seb', 200)
        @player3 = Player.new('fina', 300)

        @players = [@player1, @player2, @player3]
      end

      it 'is sorted by decreasing score' do
        expect(@players.sort).to eq([@player3, @player2, @player1])
      end
    end

  end
end
